@@ -27,8 +27,8 @@
 #endif
 
 #define JSON_VERSION_MAJOR 0
-#define JSON_VERSION_MINOR 7
-#define JSON_VERSION_PATCH 2
+#define JSON_VERSION_MINOR 8
+#define JSON_VERSION_PATCH 0
 
 #define JSON_VERSION                 \
     (                                \
@@ -307,6 +307,8 @@ struct json_type_sizes_t
 {
     struct json_obj_sizes_t obj;
     struct json_type_lib_sizes_t lib;
+    size_t var_stack_max;
+    size_t var_stack_init;
 };
 
 struct json_type_t;
